# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2018-02-14 20:53
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('apollo', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ExternalWebhook',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.CharField(max_length=255)),
                ('for_event', models.CharField(choices=[('new form submission', 'new form submission')], max_length=100)),
            ],
        ),
        migrations.AddField(
            model_name='form',
            name='allow_webhooks',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='externalwebhook',
            name='form',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='external_hooks', to='apollo.Form'),
        ),
    ]
