# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2022-08-16 20:13
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apollo', '0003_externalwebhook_service'),
    ]

    operations = [
        migrations.AddField(
            model_name='form',
            name='include_recaptcha',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='formfield',
            name='validation_rule',
            field=models.CharField(blank=True, choices=[('OPTIONAL', 'OPTIONAL'), ('REQUIRED', 'REQUIRED'), ('EMAIL', 'EMAIL'), ('PHONE_NUMBER', 'PHONE_NUMBER')], default='REQUIRED', max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='formfieldtemplate',
            name='validation_rule',
            field=models.CharField(blank=True, choices=[('OPTIONAL', 'OPTIONAL'), ('REQUIRED', 'REQUIRED'), ('EMAIL', 'EMAIL'), ('PHONE_NUMBER', 'PHONE_NUMBER')], default='REQUIRED', max_length=50, null=True),
        ),
    ]
