<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>Noscript Apollo Form</title>
    <link rel="stylesheet" type="text/css" href="{{ themes_url }}/default/index.css" />

    <style>
        .icarus-field label {
            display: block;
        }
    </style>
</head>
<body class="form-{{ form_id }}">
    <div class="icarus">
        <form class="icarus-form" method="post">{% csrf_token %}
            <div class="icarus-layout-static">
            {% for field in form.visible_fields %}
                <div class="icarus-field field-{{ field.name }}">
                    {% if field.label %}
                    <label for="{{ field.name }}">{{ field.label }}</label>
                    {% endif %}

                    {{ field }}
                </div>
            {% endfor %}

            {% for field in form.hidden_fields %}
                {{ field }}
            {% endfor %}
            </div>

            <input type="hidden" value="{{ form_id }}" name="form_id" />

            {% if error %}
            <ul class="icarus-errors">
                <li>{{ error }}</li>
            </ul>
            {% endif %}

            <button type="submit" class="submit-btn">Submit</button>
        </form>
    </div>
</body>
</html>
