# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2018-02-14 21:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apollo', '0002_auto_20180214_2053'),
    ]

    operations = [
        migrations.AddField(
            model_name='externalwebhook',
            name='service',
            field=models.CharField(default='zapier', max_length=100),
            preserve_default=False,
        ),
    ]
